==> quetoo-0.3.3.tar.bz2: quetoo-0.3.3/src/q_sh.c <==
/*
* Copyright(c) 1997-2001 Id Software, Inc.
* Copyright(c) 2002 The Quakeforge Project.
* Copyright(c) 2006 Quetoo.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or(at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
* 
* See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <stdio.h>
#include <ctype.h>
#include <dirent.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/time.h>

#include "qcommon.h"


byte *membase;
int maxhunksize;
int curhunksize;

void *Hunk_Begin(int maxsize){
	// reserve a huge chunk of memory, but don't commit any yet
	maxhunksize = maxsize + sizeof(int);
	curhunksize = 0;
	
	membase = mmap(0, maxhunksize, 
		PROT_READ | PROT_WRITE, MAP_PRIVATE | MAP_ANONYMOUS, -1, 0
	);
	
	if(membase == NULL || membase ==(byte *) - 1)
		Sys_Error("unable to virtual allocate %d bytes", maxsize);
		
	*((int *)membase) = curhunksize;
	
	return membase + sizeof(int);
}

void *Hunk_Alloc(int size){
	byte *buf;
	
	// round to cacheline
	size =(size + 31) & ~31;
	if(curhunksize + size > maxhunksize)
		Sys_Error("Hunk_Alloc overflow");
	buf = membase + sizeof(int) + curhunksize;
	curhunksize += size;
	return buf;
}

int Hunk_End(void){
	byte *n;
	
	n = mremap(membase, maxhunksize, curhunksize + sizeof(int), 0);
	
	if(n != membase)
		Sys_Error("Hunk_End:  Could not remap virtual block(%d)", errno);
	*((int *)membase) = curhunksize + sizeof(int);
	
	return curhunksize;
}

void Hunk_Free(void *base){
	byte *m;
	
	if(base){
		m =((byte *)base) - sizeof(int);
		if(munmap(m, *((int *)m)))
			Sys_Error("Hunk_Free: munmap failed(%d:%s)", errno, strerror(errno));
	}
}



/*
Sys_Milliseconds
*/
int curtime;

int Sys_Milliseconds(void){
	struct timeval tp;
	struct timezone tzp;
	static int secbase;

	gettimeofday(&tp, &tzp);

	if(!secbase){
		secbase = tp.tv_sec;
		return tp.tv_usec / 1000;
	}

	curtime =(tp.tv_sec - secbase) * 1000 + tp.tv_usec / 1000;

	return curtime;
}

void Sys_Mkdir(char *path){
	mkdir(path, 0777);
}

char *strlwr(char *s){
	char *p = s;
	while(*s){
		*s = tolower(*s);
		s++;
	}
	return p;
}


static	char findbase[MAX_OSPATH];
static	char findpath[MAX_OSPATH];
static	char findpattern[MAX_OSPATH];
static	DIR	*fdir;

/*
*  ComareAttributes, not really implemented :/
*/
static qboolean CompareAttributes(char *path, char *name,
		unsigned musthave, unsigned canthave){
	
	if(strcmp(name, ".") == 0 || strcmp(name, "..") == 0)
		return false;
	
	return true;
}

char *Sys_FindFirst(char *path, unsigned musthave, unsigned canhave){
	struct dirent *d;
	char *p;
	
	if(fdir)
		Sys_Error("Sys_BeginFind without close");
		
	strcpy(findbase, path);
	
	if((p = strrchr(findbase, '/')) != NULL){
		*p = 0;
		strcpy(findpattern, p + 1);
	} else
		strcpy(findpattern, "*");
		
	if(strcmp(findpattern, "*.*") == 0)
		strcpy(findpattern, "*");
		
	if((fdir = opendir(findbase)) == NULL)
		return NULL;
	while((d = readdir(fdir)) != NULL){
		if(!*findpattern || COM_GlobMatch(findpattern, d->d_name)){
			if(CompareAttributes(findbase, d->d_name, musthave, canhave)){
				sprintf(findpath, "%s/%s", findbase, d->d_name);
				return findpath;
			}
		}
	}
	return NULL;
}

char *Sys_FindNext(unsigned musthave, unsigned canhave){
	struct dirent *d;
	
	if(fdir == NULL)
		return NULL;
	while((d = readdir(fdir)) != NULL){
		if(!*findpattern || COM_GlobMatch(findpattern, d->d_name)){
			if(CompareAttributes(findbase, d->d_name, musthave, canhave)){
				sprintf(findpath, "%s/%s", findbase, d->d_name);
				return findpath;
			}
		}
	}
	return NULL;
}

void Sys_FindClose(void){
	if(fdir != NULL)
		closedir(fdir);
	fdir = NULL;
}

==> quetoo-0.4.0.tar.bz2: quetoo-0.4.0/src/q_sh.c <==
/*
* Copyright(c) 1997-2001 Id Software, Inc.
* Copyright(c) 2002 The Quakeforge Project.
* Copyright(c) 2006 Quetoo.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or(at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
* 
* See the GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <stdio.h>
#include <ctype.h>
#include <dirent.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/time.h>

#include "qcommon.h"

byte *hunk = NULL;
cvar_t *hunkmegs;
int hunkbytes;
int hunkofs;

void Hunk_Init(void){
	
	if(hunk)  // free the old hunk
		free(hunk);
	
	hunkmegs = Cvar_Get("hunkmegs", "16", CVAR_LATCH);
	hunkbytes = hunkmegs->value * 1024 * 1024;
	
	if(!(hunk = malloc(hunkbytes)))  // malloc the new one
		Sys_Error("Unable to allocate hunk");
	
	hunkofs = 0;
}

void *Hunk_Begin(){
	return hunk + hunkofs;
}

int Hunk_End(void *buf){
	return hunk + hunkofs - (byte *)buf;
}

void *Hunk_Alloc(int size){
	byte *b;
	
	size = (size + 31) & ~31;  // round to cacheline
	
	if(hunkofs + size > hunkbytes)
		Sys_Error("Hunk_Alloc overflow");
	
	b = hunk + hunkofs;
	hunkofs += size;
	return b;
}

void Hunk_FreeAll(void){
	memset(hunk, 0, hunkbytes);
	hunkofs = 0;
}

void Hunk_Shutdown(void){
	if(hunk)  // free the hunk
		free(hunk);
}

/*
Sys_Milliseconds
*/
int curtime;

int Sys_Milliseconds(void){
	struct timeval tp;
	struct timezone tzp;
	static int secbase;
	
	gettimeofday(&tp, &tzp);
	
	if(!secbase){
		secbase = tp.tv_sec;
		return tp.tv_usec / 1000;
	}
	
	curtime = (tp.tv_sec - secbase) * 1000 + tp.tv_usec / 1000;
	
	return curtime;
}

void Sys_Mkdir(char *path){
	mkdir(path, 0777);
}

char *strlwr(char *s){
	char *p = s;
	while(*s){
		*s = tolower(*s);
		s++;
	}
	return p;
}


static char findbase[MAX_OSPATH];
static char findpath[MAX_OSPATH];
static char findpattern[MAX_OSPATH];
static DIR *fdir;


char *Sys_FindFirst(char *path){
	struct dirent *d;
	char *p;
	
	if(fdir)
		Sys_Error("Sys_BeginFind without close");
		
	strcpy(findbase, path);
	
	if((p = strrchr(findbase, '/')) != NULL){
		*p = 0;
		strcpy(findpattern, p + 1);
	} else
		strcpy(findpattern, "*");
		
	if(strcmp(findpattern, "*.*") == 0)
		strcpy(findpattern, "*");
		
	if((fdir = opendir(findbase)) == NULL)
		return NULL;
	while((d = readdir(fdir)) != NULL){
		if(!*findpattern || COM_GlobMatch(findpattern, d->d_name)){
			sprintf(findpath, "%s/%s", findbase, d->d_name);
			return findpath;
		}
	}
	return NULL;
}

char *Sys_FindNext(){
	struct dirent *d;
	
	if(fdir == NULL)
		return NULL;
	while((d = readdir(fdir)) != NULL){
		if(!*findpattern || COM_GlobMatch(findpattern, d->d_name)){
			sprintf(findpath, "%s/%s", findbase, d->d_name);
			return findpath;
		}
	}
	return NULL;
}

void Sys_FindClose(void){
	if(fdir != NULL)
		closedir(fdir);
	fdir = NULL;
}
